#! /usr/bin/perl
#Name: R3-client.perl
#Project: R3 control
#Homepage: www.HomoFaciens.de
#Author Norbert Heinz
#Version: 1.0
#Creation date: 08.08.2012
#Copyright: (c) 2012 by Norbert Heinz
#This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/.
#
#Client script for rover control.

use strict;
use LWP::UserAgent;
use HTTP::Request::Common;
use Getopt::Long;
use File::Basename;
use LWP::Simple;

$|=1 ;

my $pic_count = 0;
my $file_in = "00000010.jpg";
my $char_in = ""; 
my $i;
my @file_info;
my $file_size = 0;
my $char_temp = "";
my $wait_count = 0;
my $driveDistance = 0;
my $cameraAngle = 0;
my $cameraAngleOld = -1;
my $steeringAngleOld = -1;
my $last_pic_send = 0;
my $pic_save_nr = time();
my $script_answer = "";
my $first_run = 1;

#Password must be identically in SendCurrentCommand.pl and fileupload.pl stored at the Internet server.
my $password = "TypeYourPassworHere";

my $url_file_upload = "http://127.0.0.1/perl/R3/fileupload.pl";
my $url_ask_command = 'http://127.0.0.1/perl/R3/SendCurrentCommand.pl?passwd=' . $password ;

$script_answer = get $url_ask_command . "&status=finished";

while (1 == 1){


    $script_answer = get $url_ask_command . "&status=waiting";   
    print "2.) script_answer=>" . $script_answer . "<, url_ask_command=>" . $url_ask_command . "<\n";

    while($script_answer eq "no job available"){
      if($wait_count == 15){
        &ExecuteCommandLine("./R3C1 o 1");                  #Initialize R3
        $steeringAngleOld = -1;
        $cameraAngleOld = -1;
      }
      if($wait_count == 20){
        &ExecuteCommandLine("./R3C1 O 1");                  #set R3 in parking position (sleep)
        $steeringAngleOld = -1;
        $cameraAngleOld = -1;
        if($first_run == 0){
          $script_answer = get $url_ask_command . "&status=finished"; 
        }
        else{
          $first_run = 0;
        }
      }

      $wait_count ++;
      print "wait_count = $wait_count\n";
      sleep(2);
      $script_answer = get $url_ask_command . "&status=waiting";   
      print "3.) script_answer=>" . $script_answer . "<\n";
    }

    if(substr($script_answer, 0, 5) eq "Comm="){
      $wait_count = 0;
      $first_run = 0;
      print "\n\nReceived command '" . $script_answer . "'\n\n";


      if(substr($script_answer, 13, 1) eq "0"){#Lights off
        &ExecuteCommandLine("./R3C1 l 0");
      }
      if(substr($script_answer, 13, 1) eq "1"){#Light low
        &ExecuteCommandLine("./R3C1 l 1");
      }
      if(substr($script_answer, 13, 1) eq "2"){
        &ExecuteCommandLine("./R3C1 l 2");
      }
      if(substr($script_answer, 13, 1) eq "3"){
        &ExecuteCommandLine("./R3C1 l 3");
      }
      if(substr($script_answer, 13, 1) eq "4"){#Light highest possible
        &ExecuteCommandLine("./R3C1 l 4");
      }

      $cameraAngle = substr($script_answer, 11, 2);
      if($cameraAngle < 0 | $cameraAngle > 15){
        print "cameraAngle=$cameraAngle out of range!\n";
        $cameraAngle = 255;
      }
      else{
        if($cameraAngle != $cameraAngleOld){
          $cameraAngleOld = $cameraAngle;
          $cameraAngle = sprintf("%.0f", 800 * $cameraAngle / 15 + 900);
          &ExecuteCommandLine("./R3C1 v $cameraAngle");
        }
      }

      if(substr($script_answer, 6, 1) eq "2" | substr($script_answer, 6, 1) eq "5"){#turn servo right
        if($steeringAngleOld != 2){
          &ExecuteCommandLine("./R3C1 s 400");
          $steeringAngleOld = 2;
        }
      }
      if(substr($script_answer, 6, 1) eq "1" | substr($script_answer, 6, 1) eq "4"){#turn servo left
        if($steeringAngleOld != 1){
          &ExecuteCommandLine("./R3C1 s 1700");
          $steeringAngleOld = 1;
        }
      }
      if(substr($script_answer, 6, 1) eq "0" | substr($script_answer, 6, 1) eq "3"){#servo straight
        if($steeringAngleOld != 0){
          &ExecuteCommandLine("./R3C1 s 1000");
          $steeringAngleOld = 0;
        }
      }

      $driveDistance = substr($script_answer, 7, 2) * 20;
      if($driveDistance < 0 | $driveDistance > 300){
        print "driveDistance=$driveDistance out of range!\n";
        $driveDistance = 0;
      }

      if(substr($script_answer, 6, 1) eq "0" | substr($script_answer, 6, 1) eq "1" | substr($script_answer, 6, 1) eq "2"){#Driving forward
        &ExecuteCommandLine("./R3C1 f $driveDistance");
      }
      else{
        &ExecuteCommandLine("./R3C1 b $driveDistance");
      }

      #Use mplayer or ffmpeg to get a single image from the webcam. several pics are generated to give the wabcam some time to autoadjust the brightness.

      `killall -s SIGKILL ffmpeg`;
      `killall -s SIGKILL mplayer`;
      #&ExecuteCommandLine("mplayer tv:// -tv driver=v4l2:device=/dev/video0:width=320:height=180:outfmt=rgb24:quality=50 -frames 15 -vo jpeg");
      &ExecuteCommandLine("ffmpeg -f video4linux2 -s 320x180 -r 5 -t 4 -vcodec mjpeg -i /dev/video0 %08d.jpg");

      #Open image file and start transfer to server
      # put timeouts, proxy etc into the useragent if needed
      print $url_file_upload . "\n";
      my $ua  = LWP::UserAgent->new();
      my $req = POST $url_file_upload, Content_Type => 'form-data', Content => [submit => 1, myfile => [ $file_in ], passwd => $password];
      my $response = $ua->request($req);

      if ($response->is_success()) {
          print "OK: ", $response->content;
      }
      else {
          print "ERROR: " . $response->as_string;
      }

      $pic_count++;

      print "File no. " . $pic_count . " sent\n";

      print "Command finished...\n";
      $script_answer = get $url_ask_command . "&status=finished"; 
    }#if(substr($script_answer, 0, 5) eq "Comm="){
    else{
      die "Invalid commandline! >$script_answer<, probably incorrect password.\n"
    }

}#while (1 == 1)


print $pic_count . " files sent\n";

#++++++++++++++++++++++ ExecuteCommandLine ++++++++++++++++++++++++++++++++++++++++++++++++++
sub ExecuteCommandLine{
  print $_[0] . "\n";
  `$_[0]`;
}


